@@ -35,6 +35,10 @@
 #include <gdk/gdkkeysyms.h>
 #include <libsn/sn-launchee.h>
 
+/* Parameters for the ellipsization in the Tabs menu */
+#define TAB_MENU_WIDTH_CHARS    35
+#define TAB_MENU_ELLIPSIZE      PANGO_ELLIPSIZE_END
+
 struct _TerminalWindowPrivate
 {  
   GtkWidget *main_vbox;
@@ -423,8 +427,8 @@ fill_in_config_picker_submenu (TerminalWindow *window)
   if (window->priv->active_term == NULL || g_list_length (profiles) < 2)
     {
       gtk_widget_set_sensitive (window->priv->choose_config_menuitem, FALSE);
-      gtk_menu_item_set_submenu (GTK_MENU_ITEM (window->priv->choose_config_menuitem),
-                                 NULL);
+      gtk_menu_item_remove_submenu (GTK_MENU_ITEM (window->priv->choose_config_menuitem));
+
       return;
     }
   
@@ -518,8 +522,7 @@ fill_in_new_term_submenu_real(GtkWidget *menuitem,
     {
       /* Well, this shouldn't happen: it'd mean there is no default profile */
       gtk_widget_set_sensitive (window->priv->new_window_menuitem, FALSE);
-      gtk_menu_item_set_submenu (GTK_MENU_ITEM (window->priv->new_window_menuitem),
-                                 NULL);
+      gtk_menu_item_remove_submenu (GTK_MENU_ITEM (window->priv->new_window_menuitem));
     }
   else if (g_list_length (profiles) < 2)
     {
@@ -637,8 +640,8 @@ fill_in_encoding_menu (TerminalWindow *window)
   if (window->priv->active_term == NULL)
     {
       gtk_widget_set_sensitive (window->priv->encoding_menuitem, FALSE);
-      gtk_menu_item_set_submenu (GTK_MENU_ITEM (window->priv->encoding_menuitem),
-                                 NULL);
+      gtk_menu_item_remove_submenu (GTK_MENU_ITEM (window->priv->encoding_menuitem));
+
       return;
     }
   
@@ -1071,7 +1074,7 @@ terminal_window_init (TerminalWindow *window)
   mi = gtk_separator_menu_item_new ();
   gtk_menu_shell_append (GTK_MENU_SHELL (menu), mi);
 
-  mi = append_menuitem (menu, _("Detach Tab"), ACCEL_PATH_DETACH_TAB,
+  mi = append_menuitem (menu, _("_Detach Tab"), ACCEL_PATH_DETACH_TAB,
                         G_CALLBACK (detach_tab_callback), window);
   window->priv->detach_tab_menuitem = mi;
 
@@ -1158,9 +1161,6 @@ terminal_window_destroy (GtkObject *object)
 
   window = TERMINAL_WINDOW (object);
 
-  while (window->priv->terms)
-    terminal_window_remove_screen (window, window->priv->terms->data);
-
   g_list_free (window->priv->tab_menuitems);
   window->priv->tab_menuitems = NULL;
   
@@ -1343,22 +1343,34 @@ static void
 title_changed_callback (TerminalScreen *screen,
                         TerminalWindow *window)
 {
-  GtkWidget *mi;
+  GtkWidget *menu_item;
+  const char *title;
   
-  if (screen == window->priv->active_term) 
+  title = terminal_screen_get_title (screen);
+
+  menu_item = screen_get_menuitem (screen);
+  if (menu_item)
     {
-    gtk_window_set_title (GTK_WINDOW (window),
-                          terminal_screen_get_title (screen));
-    if (terminal_screen_get_icon_title_set (screen))
-      gdk_window_set_icon_name (GTK_WIDGET (window)->window, terminal_screen_get_icon_title (screen));
-    else
-      gdk_window_set_icon_name (GTK_WIDGET (window)->window, terminal_screen_get_title (screen));
+      GtkWidget *label;
+
+      label = gtk_bin_get_child (GTK_BIN (menu_item));
+
+      gtk_label_set_use_underline (GTK_LABEL (label), FALSE);
+      gtk_label_set_ellipsize (GTK_LABEL (label), TAB_MENU_ELLIPSIZE);
+      gtk_label_set_max_width_chars (GTK_LABEL (label), TAB_MENU_WIDTH_CHARS); 
+      gtk_label_set_text (GTK_LABEL (label), title);
     }
 
-  mi = screen_get_menuitem (screen);
-  if (mi)
-    gtk_label_set_text (GTK_LABEL (gtk_bin_get_child (GTK_BIN (mi))),
-                        terminal_screen_get_title (screen));
+  if (screen == window->priv->active_term) 
+    {
+      gtk_window_set_title (GTK_WINDOW (window), title);
+
+      if (terminal_screen_get_icon_title_set (screen))
+        {
+          title = terminal_screen_get_icon_title (screen);
+        }
+      gdk_window_set_icon_name (GTK_WIDGET (window)->window, title);
+    }
 }
 
 static void
@@ -1617,8 +1629,22 @@ terminal_window_set_active (TerminalWindow *window,
   if (window->priv->active_term == screen)
     return;
   
+  /* Workaround to remove gtknotebook's feature of computing its size based on
+   * all pages. When the widget is hidden, its size will not be taken into
+   * account.
+   */
+  if (window->priv->active_term)
+  {
+    GtkWidget *old_widget;
+    old_widget = terminal_screen_get_widget (window->priv->active_term);
+    gtk_widget_hide (old_widget);
+  }
+  
   widget = terminal_screen_get_widget (screen);
   
+  /* Make sure that the widget is no longer hidden due to the workaround */
+  gtk_widget_show (widget);
+
   profile = terminal_screen_get_profile (screen);
 
   if (!GTK_WIDGET_REALIZED (widget))
@@ -1709,11 +1735,7 @@ notebook_page_selected_callback (GtkWidget       *notebook,
   TerminalScreen *screen;
   GtkWidget *menu_item;
   int old_grid_width, old_grid_height;
-  GtkWidget *old_widget;
-  
-  old_widget = NULL;
-  old_grid_width = -1;
-  old_grid_height = -1;
+  GtkWidget *old_widget, *new_widget;
 
   if (window->priv->active_term == NULL)
     return;
@@ -1729,17 +1751,13 @@ notebook_page_selected_callback (GtkWidget       *notebook,
   screen = TERMINAL_SCREEN (page_widget);
 
   g_assert (screen);
+  
+  /* This is so that we maintain the same grid */
+  new_widget = terminal_screen_get_widget (screen);
+  terminal_widget_set_size (new_widget, old_grid_width, old_grid_height);
 
   terminal_window_set_active (window, screen);
 
-  /* This is so we maintain the same grid moving among tabs with
-   * different fonts.
-   */
-#ifdef DEBUG_GEOMETRY
-  g_fprintf (stderr,"setting size in switch_page handler\n");
-#endif
-  terminal_window_set_size_force_grid (window, screen, TRUE, old_grid_width, old_grid_height);
-  
   update_tab_sensitivity (window);
   
   menu_item = screen_get_menuitem (screen);
@@ -1803,20 +1821,20 @@ notebook_tab_added_callback (GtkWidget       *notebook,
    */
   gtk_widget_realize (GTK_WIDGET (term));
 
-  {
-    /* Match size to current active screen */
-    int current_width, current_height;
-    if (window->priv->active_term)
-      terminal_widget_get_size (terminal_screen_get_widget(window->priv->active_term), 
-                                &current_width, &current_height);
-    else
-      {
-        current_width = 80;
-        current_height = 24;
-      }
+  /* If we have an active screen, match its size and zoom */
+  if (window->priv->active_term)
+    {
+      GtkWidget *widget;
+      int current_width, current_height;
+      double scale;
 
-    terminal_widget_set_size (term, current_width, current_height);
-  }
+      widget = terminal_screen_get_widget (window->priv->active_term);
+      terminal_widget_get_size (widget, &current_width, &current_height);
+      terminal_widget_set_size (term, current_width, current_height);
+
+      scale = terminal_screen_get_font_scale (window->priv->active_term);
+      terminal_screen_set_font_scale (screen, scale);
+    }
   
   /* Make the first-added screen the active one */
   if (window->priv->active_term == NULL)
@@ -2132,6 +2150,7 @@ reset_tab_menuitems (TerminalWindow *window)
   while (TRUE) /* should probably make us somewhat nervous */
     {
       GtkWidget *page;
+      GtkWidget *label;
       char *accel_path;
       
       page = gtk_notebook_get_nth_page (GTK_NOTEBOOK (window->priv->notebook),
@@ -2144,6 +2163,11 @@ reset_tab_menuitems (TerminalWindow *window)
 
       menu_item = gtk_radio_menu_item_new_with_label (group,
                                                       terminal_screen_get_title (screen));
+      label = gtk_bin_get_child (GTK_BIN (menu_item));
+      gtk_label_set_use_underline (GTK_LABEL (label), FALSE);
+      gtk_label_set_ellipsize (GTK_LABEL (label), TAB_MENU_ELLIPSIZE);
+      gtk_label_set_max_width_chars (GTK_LABEL (label), TAB_MENU_WIDTH_CHARS); 
+
       group = gtk_radio_menu_item_get_group (GTK_RADIO_MENU_ITEM (menu_item));
       
       if (i < N_TABS_WITH_ACCEL && !single_page)
@@ -2243,13 +2267,31 @@ new_window (TerminalWindow *window,
             char *name,
             const char *dir)
 {
+  char *geometry;
+
+  if (screen)
+    {
+      GtkWidget *term;
+      int width, height;
+
+      term = terminal_screen_get_widget (screen);
+      terminal_widget_get_size (term, &width, &height);
+      geometry = g_strdup_printf("%dx%d", width, height);
+    }
+  else
+    {
+      geometry = NULL;
+    }
+
   terminal_app_new_terminal (terminal_app_get (),
                              profile,
                              NULL,
                              screen,
                              FALSE, FALSE, FALSE,
-                             NULL, NULL, NULL, dir, NULL, 1.0,
+                             NULL, geometry, NULL, dir, NULL, 1.0,
                              NULL, name, -1);
+
+  g_free (geometry);
 }
 
 static void
@@ -2305,8 +2347,7 @@ confirm_close_window (TerminalWindow *window)
 			  		    ngettext ("This window has one tab open. Closing "
 						      "the window will close it.",
 						      "This window has %d tabs open. Closing "
-						      "the window will also close all of its "
-						      "tabs.",
+						      "the window will also close all tabs.",
 						      n),
 					    n);
 
@@ -2663,6 +2704,8 @@ accel_event_key_match (GdkEventKey *event, GtkAccelKey *key)
 
   if (modifiers & GDK_LOCK_MASK)
     modifiers -= GDK_LOCK_MASK;
+  if (modifiers & GDK_MOD2_MASK)
+    modifiers -= GDK_MOD2_MASK;
 
   if (modifiers != key->accel_mods)
     return FALSE;
